<?php
namespace Controllers;

// Контроллер для чеков
class Receipts extends Controller {
	public $products, $changes, $receiptlines, $receipts_types;

	public function __construct() {
		parent::__construct();
		$this->model=new \Models\Receipts($this->db, $this->f3);
		$this->receiptlines=new \Models\Receiptlines($this->db, $this->f3);
		$this->products=new \Models\Products($this->db);
		$this->changes=new \Models\Changes($this->db, $this->f3);
		$this->receipts_types=array(
			'sales'=>$this->f3->get('receipt_sales'),
			'return'=>$this->f3->get('receipt_return'),
			'cashin'=>$this->f3->get('receipt_cashin'),
			'cashout'=>$this->f3->get('receipt_cashout')
		);
	}

	// Раздел продаж
	function sales() {
		if(!$this->f3->get('return') && !$this->f3->get('AJAX')) {
			$this->cd->seller_txt();
		}
		if($this->f3->get('return')) {
			$this->cd->close_txt();
		}
		$this->f3->set('page_data', $this->model->page_data('sales'));
	}

	// Раздел возрата
	function return() {
		$id=$this->f3->get('GET.id');
		if(!$id) {
			$this->f3->set('return', 1);
			$this->sales();
			$this->f3->set('page_data["title"]', $this->model->page_data('return')['title']);
		}
		else {
			$receipt=$this->model->select_receipts(array('r.id=? AND r.type=1', $id), array('limit'=>1));
			if($receipt) {
				$p=new \Models\Paymentmethods($this->db, $this->f3);
				$paymentmethods=$p->select_method(null);
				foreach ($paymentmethods as $n => $p) {
					$pays[$p['id']]=$p['title'];
				}
				$this->f3->set('paymentmethods', $pays);
				$this->f3->set('receipt', $receipt[0]);
				$this->f3->set('page_data', $this->model->page_data('return'));
			}
			else {
				$this->f3->set('page_data', $this->messages->message('not_found'));
			}
		}
	}

	// Поиск чеков
	function search() {
		$id=$this->f3->get('GET.id');
		$changes=$this->f3->get('GET.changes');
		$barcode=$this->f3->get('GET.barcode');
		$s=new \Models\Settings($this->f3, $this->db);
		$sets=$s->get_settings(array('weight_code_pr', 'card_code_pr'));
		// Проверяем штрих-код на принадлежность к весовому типу
		if(mb_substr($barcode, 0, 2)==$sets['weight_code_pr'] && mb_strlen($barcode)==13) {
			$barcode=mb_substr($barcode, 0, 7);
		}
		$dates=$this->f3->get('GET.dates');
		$type=$this->f3->get('GET.type');
		if($dates) {
			$dates=str_replace(' ', '', $dates);
			$dates=explode('-', $dates);
			$dates[0]=strtotime($dates[0]);
			$dates[1]=strtotime($dates[1]);
		}
		$limit=$this->f3->get('lines_of_page');
		$page=$this->f3->get('GET.page');
		if(!$page)$page=1;
		$receipts=$this->model->search($id, $changes, $barcode, $dates, $type, ($page-1)*$limit, $limit);
		$this->f3->set('pagination', $this->pagination($receipts['count'], $page));
		$this->f3->set('receipts', $receipts['receipts']);
		$this->f3->set('page_data', $this->model->page_data('search'));
	}

	// Просмотр чека
	function show() {
		$id=$this->f3->get('GET.id');
		$page_data=$this->messages->message('not_found');
		if($id) {
			$receipts=$this->model->select_receipts(array('r.id=?', $id), array('limit'=>1));
			if($receipts) {
				$p=new \Models\Paymentmethods($this->db, $this->f3);
				$paymentmethods=$p->select_method();
				foreach ($paymentmethods as $n => $p) {
					$pays[$p['id']]=$p['title'];
				}
				$this->f3->set('paymentmethods', $pays);
				$fr=new \Models\Fr($this->db, $this->f3);
				$sets=$fr->get_fr();
				$this->f3->set('fr', $sets);
				$this->f3->set('receipt', $receipts[0]);
				$page_data=$this->model->page_data('show');
			}
		}
		$this->f3->set('page_data', $page_data);
	}

	// добавление строки в чек
	function add_line() {
		$result=array(
			'success'=>false,
			'txt'=>$this->messages->message('not_found')['txt']
		);
		$product_id=$this->f3->get('GET.product_id');
		$count=$this->f3->get('GET.count');
		if(!$count) $count=1;
		if($product_id) {
			if($product_id>0) {
				$res=$this->products->select_product(array('p.id=?', $product_id), array('limit'=>1));
			}
			else {
				$res[0]=array(
					'title'=>'Свободная продажа',
					'price'=>$this->f3->get('GET.price'),
					'vat'=>0,
					'vat_type'=>0
				);
			}
			if(!empty($res)) {
				$result=$this->receiptlines->add_line($res[0], $count);
				if($result['success']) {
					$this->cd->print_line($res[0], $count);
				}
				$result['txt']=$this->messages->message($result['txt'])['txt'];
			}
		}

		echo json_encode($result);
		exit();
	}

	// Меняем строку в чеке
	function change_line() {
		$line_id=$this->f3->get('REQUEST.line_id');
		$count=$this->f3->get('REQUEST.count');
		$price=$this->f3->get('REQUEST.price');
		$discount=$this->f3->get('REQUEST.discount_line');
		if($line_id>=0 && ($count>=0 || $price>=0)) {
			$res=$this->receiptlines->change_line($line_id, $count, $price, $discount);
			if($res['success']) {
				$line=$this->f3->get('SESSION.receiptlines')[$line_id];
				$this->cd->print_line($line, $line['count']);
			}
			$res['txt']=$this->messages->message($res['txt'])['txt'];
		}
		else {
			$res=array(
				'success'=>false,
				'txt'=>$this->messages->message('not_enough_data')['txt']
			);
		}

		echo json_encode($res);
		exit();
	}

	// Редактируем цену в чеке
	function change_price() {
		$line_id=$this->f3->get('GET.line_id');
		$lines=$this->f3->get('SESSION.receiptlines');
		if($lines[$line_id]) {
			$this->f3->set('line', $lines[$line_id]);
			$this->f3->set('page_data', $this->model->page_data('change_price'));
		}
		else {
			$this->f3->set('page_data', $this->messages->message('not_enough_data'));
		}
	}

	// Сохраняем новую цену продажи
	function post_change_price() {
		$this->change_line();
	}

	// Удаляем строки
	function delete_line() {
		$this->cd->clear_cd();
		$line_id=$this->f3->get('GET.line_id');
		if($line_id>=0 || $line_id=='all') {
			$res=$this->receiptlines->delete_line($line_id);
			$res['txt']=$this->messages->message($res['txt'])['txt'];
		}
		else {
			$res=array(
				'success'=>false,
				'txt'=>$this->messages->message('not_enough_data')['txt']
			);
		}

		echo json_encode($res);
		exit();
	}

	// выбор оплаты
	function pay() {
		$sum=$this->f3->get('GET.sum');
		$pay=$this->f3->get('GET.pay');
		if($sum && $pay) {
			$this->cd->total_price($sum, $pay);
			exit();
		}
		$lines=$this->f3->get('SESSION.receiptlines');
		if(!$lines) {
			$this->f3->set('page_data', $this->messages->message('not_enough_data'));
			return;
		}
		$discount=$this->f3->get('COOKIE.discount');
		$sum=$this->receiptlines->get_sum($lines, $discount);
		$this->f3->set('sum', $sum);
		$p=new \Models\Paymentmethods($this->db, $this->f3);
		$paymentmethods=$p->select_method();
		$type=$this->f3->get('GET.type');
		if($type!=$this->receipts_types['return']) $type=$this->receipts_types['sales'];
		if($type==$this->receipts_types['sales']) {
			$this->cd->total_price($sum, $sum);
		}
		$this->f3->set('type', $type);
		$this->f3->set('paymentmethods', $paymentmethods);
		$this->f3->set('page_data', $this->model->page_data('paymentmethods'));
	}

	// Пробиваем чек
	function ready($lines) {
		$paymentmethod=$this->f3->get('REQUEST.paymentmethod');
		$client_sum=$this->f3->get('REQUEST.client_sum');
		$client=$this->f3->get('REQUEST.client');
		$sales_receipts=$this->f3->get('REQUEST.sales_receipts');
		if($lines && $paymentmethod && $client_sum) {
			$discount=$this->f3->get('POST.discount');
			if(!isset($discount)) $discount=$this->f3->get('COOKIE.discount');
			$type=$this->f3->get('REQUEST.type');
			if(!$type) $type=$this->receipts_types['sales']; // Продажа
			$partner_id=$this->f3->get('POST.partner_id');
			if(!isset($partner_id)) {
				$partner_id=$this->f3->get('COOKIE.partner_id');
			}
			$sum=$this->receiptlines->get_sum($lines, $discount);
			// Узнаем смену
			$change=$this->changes->get_change();
			if(!empty($change)) {
				// Создаем чек
				$api=$this->controller->get_api_config();
				$res=$this->model->add_ticket($api, $change, $type, $paymentmethod, $client_sum, $sum, $lines, $client, $discount, $partner_id, $sales_receipts);
				// Добавляем товары в чек
				if($res['success']) {
					$id=$res['id'];
					$this->model->tickets_end($id);
					$res=$this->receiptlines->save_lines($id, $lines, $discount);
					// Если не удалось добавить товар, то удаляем весь чек
					if(!$res['success']) {
						$this->model->delete_receipt($id);
					}
					else {
						// Перезаписываем время чека, что подтверждает, что весь чек сохранен полностью
						$res=$this->model->ready($id);
						if($res['success']) {
							if($this->f3->get('SESSION.receiptlines')) {
								$res=$this->receiptlines->delete_line('all');
							}
						}
						else {
							$this->model->delete_receipt($id);
						}
					}
					if($type==1) {
						$this->cd->seller_txt();
					}
				}
				$res['txt']=$this->messages->message($res['txt'])['txt'];
			}
			else {
				$res=array(
					'success'=>false,
					'txt'=>$this->messages->message('change_not_open')['txt']
				);
			}
		}
		else {
			$res=array(
				'success'=>false,
				'txt'=>$this->messages->message('not_enough_data')['txt']
			);
		}

		echo json_encode($res);
		exit();
	}

	// Создаем чек продажи
	function post_sales() {
		$lines=$this->f3->get('SESSION.receiptlines');
		$this->ready($lines);
	}

	// Создаем возвратный чек
	function post_return() {
		$post=$this->f3->get('POST');
		$this->ready($post['receiptlines']);
	}

	// Внесение денег в кассу
	function cashin() {
		$this->f3->set('auto_cashout', $this->model->fr->sets['auto_incassation']);
		$this->f3->set('page_data', $this->model->page_data('cashin'));
	}

	// Изъятие денег из кассы
	function cashout() {
		$this->f3->set('auto_cashout', $this->model->fr->sets['auto_incassation']);
		$this->f3->set('page_data', $this->model->page_data('cashout'));
	}

	// Пытаемся внести деньги в кассу
	function post_cashin() {
		$sum=$this->f3->get('POST.sum');
		$description=$this->f3->get('POST.description');
		$create_receipt=$this->f3->get('POST.create_receipt');
		if($sum) {
			if($sum>0)$type=$this->receipts_types['cashin'];
			else $type=$this->receipts_types['cashout'];
			$change=$this->changes->get_change();
			if($change) {
				$api=$this->controller->get_api_config();
				$p=new \Models\Paymentmethods($this->db, $this->f3);
				$pay=$p->select_method('cash=1');
				$res=$this->model->cashin($api, $change, $pay[0]['id'],  $sum, $type, $create_receipt, $description);
				$res['txt']=$this->messages->message($res['txt'])['txt'];
			}
			else {
				$res=array(
					'success'=>false,
					'txt'=>$this->messages->message('change_not_open')['txt']
				);
			}
		}
		else {
			$res=array(
				'success'=>false,
				'txt'=>$res=$this->messages->message('not_enough_data')['txt']
			);
		}

		echo json_encode($res);
		exit();
	}

	// Изымаем деньги из кассы
	function post_cashout() {
		$this->f3->set('POST.sum', -$this->f3->get('POST.sum'));
		$this->post_cashin();
	}

	// Страница смены и отчеты
	function reports() {
		$ch=$this->f3->get('GET.change');
		if(!$ch) {
			$change=$this->changes->get_change(false);
		}
		else {
			$ch=$this->changes->load(array('id=?', $ch));
			if(!$ch->dry()) {
				$change=$ch->cast();
			}
			else {
				$this->f3->set('error', $this->messages->message('not_found')['txt']);
			}
		}
		if($change) {
			$report=$this->model->change_statistic($change['id']);
			$p=new \Models\Paymentmethods($this->db, $this->f3);
			$paymentmethods=$p->select_method();
			foreach ($paymentmethods as $p) {
				$pays[$p['id']]=array('title'=>$p['title'], 'cash'=>$p['cash']);
			}
			$this->f3->set('paymentmethods', $pays);
		}
		$report=$this->model->change_statistic($change['id']);
		$this->f3->set('report', $report);
		$this->f3->set('change', $change);
		$this->f3->set('page_data', $this->model->page_data('reports'));
	}

	// Убрать клиента из чека
	function remove_partner() {
		$p=new \Models\Partners($this->db, $this->f3);
		$res=$p->remove_partner();
		echo json_encode($res);
		exit();
	}

	// Отчет по секциям
	function rep_sections() {
		$res=$this->model->rep_sections();
		$this->f3->set('page_data', $this->messages->message($res['txt']));
	}

	// Печать копии последнего чека
	function print_last() {
		$res=$this->model->print_last();
		$res['txt']=$this->messages->message($res['txt'])['txt'];
		echo json_encode($res);
		exit();
	}
}
