<?php
namespace Models;

// Модель для всех контроллеров
class Controller {
	public $f3;

	public function __construct($f3) {
		$this->f3=$f3;
	}

	// получение данных дл подключения к API
	function get_api_config() {
		if(file_exists('app/api_config.php')) {
			include('app/api_config.php');
		}
		else {
			$api_store_id='';
			$api_rm_id='';
			$api_domen='';
			$api_pass='';
			$api_count='';
		}
		return array(
			'store_id'=>$api_store_id,
			'rm_id'=>$api_rm_id,
			'domen'=>$api_domen,
			'pass'=>$api_pass,
			'count'=>$api_count
		);
	}

	// Создание базы данных
	function create_database($instance, $db_name) {
		$api_data=$this->get_api_config(); // получаем данные для API
		// Берем по API даные о пользовательских группах и пользователях

		$api=new \Models\Api($this->f3, $api_data['domen'], $api_data['pass'], $api_data['store_id'], $api_data['count']);
		
		$data=$api->users();

		if(!$data['success']) {
			$txt='';
			foreach ($data['error'] as $er) {
				$txt.=$er.'<br>';
			}
			return array(
				'success'=>false,
				'txt'=>$txt
			);
		}

		$pays=$api->paymentmethods();

		if(!$pays['success']) {
			$txt='';
			foreach ($pays['error'] as $er) {
				$txt.=$er.'<br>';
			}
			return array(
				'success'=>false,
				'txt'=>$txt
			);
		}

		// Если нет папки db, то создаем ее 
		if(!file_exists('db')) {
			mkdir('db', 0777);
			chmod('db', 0777);
		}

		$db=new \DB\SQL('sqlite:'.$db_name);
		$sql=$this->f3->read($instance);
		$sql=str_replace("\r", "", $sql);
		$strs=explode("\n", $sql); // читаем исходник базы построчно
		foreach ($strs as $s) {
			$res+=$db->exec(trim($s)); // Создаем таблицы
		}

		if($res>0) {
			$g=new \Models\Usergroups($db, $this->f3);
			foreach ($data['usergroups'] as $group) {
				$res=$g->save_group($group, 'add');
				if(!$res['success']) {
					break;
				}
			}
			
			if($res['success']) {
				$u=new \Models\Users($db, $this->f3);
				foreach ($data['users'] as $user) {
					$res=$u->save_user($user, 'add');
					if(!$res['success']) {
						break;
					}
				}
			}

			if($res['success']) {
				foreach ($pays['paymentmethods'] as $pay) {
					$pays_type[$pay['id']]=max(abs($pay['cash']-1), 0);
					$r=$db->exec("INSERT INTO paymentmethods (id, title, cash) VALUES (?, ?, ?)", array($pay['id'], $pay['title'], $pay['cash']));
					if(!$r) {
						$res['success']=false;
						break;
					}
					else {
						$res['success']=true;
					}
				}
				$sno_api=$api->sno();
				if($sno_api['success']) {
					$sno=$sno_api['sno'];
				}
				else {
					$sno='01';
				}
				$db->exec("UPDATE fr_settings SET pays_type=?, sno=?", array(json_encode($pays_type), $sno));

				$barcodes_pr_api=$api->barcodes_pr();
				if($barcodes_pr_api['success']) {
					$sql=array();
					$val=array();
					foreach ($barcodes_pr_api['barcodes_pr'] as $name => $value) {
						$sql[]="INSERT OR REPLACE INTO settings (name, value) VALUES (?, ?)";
						$val[]=array($name, $value);
					}
					$db->exec($sql, $val);
				}
			}
		}

		if($res['success']) {
			return array(
				'success'=>true,
				'db'=>$db
			);
		}
		else {
			unset($db);
			unlink($this->f3->get('ROOT').'/'.$db_name); // удаляем файл базы, если не удалось выполнить запросы
			return array(
				'success'=>false,
				'txt'=>'db_error'
			);
		}
	}

	// Проверка доступа к разделу
	function access() {
		$user=$this->f3->get('SESSION.user');
		$page=$this->f3->get('PARAMS.model').'/'.$this->f3->get('PARAMS.method');
		if(($user['module'] && in_array($page, $user['module'])) || (($user && !$user['module'])) || !file_exists($this->f3->get('db_name')) ) {
			return array('success'=>true);
		}

		return array(
			'success'=>false,
			'txt'=>'access_closed'
		);
	}

	// Главное меню
	function get_main_menu($logs_n) {
		$menu=array(
			'/'=>array('title'=>'Продажа', 'class'=>''),
			'receipts/return'=>array('title'=>'Возврат', 'class'=>''),
			'/receipts/search'=>array('title'=>'Чеки', 'class'=>''),
			'/receipts/cashin'=>array('title'=>'Внесение', 'class'=>''),
			'/receipts/cashout'=>array('title'=>'Изъятие', 'class'=>''),
			'/receipts/reports'=>array('title'=>'Смены и отчеты', 'class'=>'')
		);

		$logs_txt='';
		if($logs_n) {
			$logs_txt=' ('.$logs_n.')';
		}
		$settings=array(
			'/settings/base'=>array('title'=>'Настройки программы', 'class'=>''),
			'/settings/api'=>array('title'=>'Настройки API', 'class'=>''),
			'/settings/kkm'=>array('title'=>'Настройки ККМ', 'class'=>''),
			'/settings/customer_display'=>array('title'=>'Настройки дисплея покупателя', 'class'=>''),
			'/settings/synchronization'=>array('title'=>'Синхронизация', 'class'=>''),
			'/settings/logs'=>array('title'=>'Ошибки при синхронизации <span id="logs_n">'.$logs_txt.'</span>', 'class'=>''),
			'/settings/reset_ticket'=>array('title'=>'Аннулирование чека', 'class'=>'show_window'),
			'/settings/continue_print'=>array('title'=>'Выход из аварийного состояния ККМ', 'class'=>'show_window'),
			'/settings/service'=>array('title'=>'Сервисное обслуживание', 'class'=>'')
		);
		
		return array(
			'menu'=>$menu,
			'settings'=>$settings
		);
	}
}
