<?php
namespace Models;

// Модель Пользователи
class Users extends \DB\SQL\Mapper {
	public $db, $f3;

	public function __construct($db, $f3) {
		parent::__construct($db, 'users' ); // подключаемся к таблицt пользователей
		$this->db=$db;
		$this->f3=$f3;
	}

	// данные о страницах
	function page_data($page) {
		$pages=array(
			'autorization'=>array(
				'title'=>'Авторизация пользователя',
				'inc'=>'users_autorization.htm'
			)
		);
		$pages[$page]['page']=$page;

		return $pages[$page];
	}

	// Выбор из базы
	function select_users($query=null, $options=null) {
		if(!$query) {
			$query=array('u.id>?', 0); // чтобы хоть что-то вставить в фильтры
		}

		// формируем сортировку и лимиты
		$option='';
		if($options['order']) {
			$option.=' ORDER BY '.$options['order'];
		}
		if($options['limit']) {
			$option.=' LIMIT '.$options['limit'];
		}
		// собственно сам запрос
		$sql="SELECT u.*, g.title AS `group_title`, g.module FROM `users` AS `u` JOIN `usergroups` AS `g` ON g.id=u.group_id WHERE date_delete=0 AND ".$query[0].$option;
		$res=$this->db->exec($sql, $query[1]);
		return $res;
	}

	// проверка на авторизацию
	function is_login() {
		$res=array(); // массив с информацией об аторизованном пользователе
		$user=$this->f3->get('SESSION.user'); // берем данные из сессии
		if(!$user) return false;
		$res=$this->select_users(array("u.id=?", array($user['id'])), array('limit'=>1));
		if(!empty($res) && $res[0]['pass'] && $user['pass']!=$res[0]['pass']) {
			return false;
		}

		if($res[0]['module']) {
			$res[0]['module']=json_decode($res[0]['module'], true);
		}
		$this->f3->set('SESSION.user', $res[0]); // перезапишем информацию об авторизации
		return true;
	}

	// авторизация пользователя
	function user_autorization($post) {
		if(!isset($post['user_id']) || !isset($post['user_pass'])) { // проверяем наличие всех нужнух данных
			return array( // если чего-то не хватает, то сообщаем об ошибке
				'success'=>false,
				'txt'=>'not_enough_data'
			);
		}

		$res=$this->select_users(array('u.id=?', $post['user_id']), array('limit'=>1)); // пытаемся начти выбранного пользователя
		if(empty($res)) { // если его нет, то выводим ошибку
			return array(
				'success'=>false,
				'txt'=>'user_not_found'
			);
		}

		if($res[0]['pass']!='' && $post['user_pass']!=$res[0]['pass']) { // проверяем пароли
			return array(
				'success'=>false,
				'txt'=>'pass_incorrect'
			);
		}
		if($res[0]['module']) {
			$res[0]['module']=json_decode($res[0]['module'], true);
		}

		$this->f3->set('SESSION.user', $res[0]); // если все ок, то запоминаем информацию о пользователе
		return array(
			'success'=>true,
			'txt'=>'login_successful',
			'reroute'=>'/'
		);
	}

	// Выход из системы
	function logout() {
		$this->f3->set('SESSION.user', '');
		$this->f3->clear('SESSION.user');
		return array(
			'success'=>true,
			'txt'=>'logout_successful',
			'reroute'=>'/users/autorization'
		);
	}

	// Сохранение пользователя
	function save_user($data, $action='add') {
		if(!isset($data['name']) || !isset($data['group_id'])) {
			return array(
				'success'=>false,
				'txt'=>'not_enough_data'
			);
		}
		$this->reset();

		if($action=='change' && $data['id']) {
			$this->load(array('id=?', $data['id']));
		}

		$this->copyFrom($data);
		$this->save();
		if($this->dry()) {
			return array(
				'success'=>false,
				'txt'=>'db_error'
			);
		}
		$this->reset();
		return array(
			'success'=>true,
			'txt'=>'save_successful'
		);
	}
}
